
/*****************************************************************************
* CT-N.ino 
* RS485	example for the CT-N series Infrared thermometer
* This example reads from the CT-N and prints out object&ambient. 
* Serial Monitor setting : 9600 bps.(Ctrl + Shift + M )

* Project Name: CT-N Series RS485 Modbus-RTU Master Code (using SoftwareSerial)
* Version: 1.0.0 
* BOARD: Arduino UNO 
* Hardware required : CT-N series, RS485-IC
* Support list : CT-200N-485, CT-300N-485, CT-1000N-485
* Development environment specifics: Arduino 1.6.4

Hardware Hookup (Please refer to the datasheet for more detailed information)
____________________________________________________________________________
 Arduino UNO  -------  | SP485 (8pin) |  --------  CT-N series (5pin)
----------------------------------------------------------------------------
   VDD(5V) ------------|--- Vcc(8) ---|----------------- VDD(1)
   RX(3) --------------|--- RO (1)    | 
   TX(5) --------------|--- DI (4)    |
   DE/RE(2) -----------|-- DE/RE(2,3) |
   GND ----------------|--- GND(5) ---|----------------- GND(2)
                       |     A(6) ----|----------------- D+(4)
                       |     B(7) ----|----------------- D-(3)
                       
List of libraries to install: SoftwareSerial, 
                              Mobus-Master-Slave-for-Arduino-Master, MsTimer2,

Revision history.
1.0.0   May 4 2022.: First version is released.

****************************************************************************/
#include <MsTimer2.h>                                 // Timer library.
#include <ModbusRtu.h>                                 // Modbus library.
#include "SoftwareSerial.h"                            // SoftwareSerial library
#define DISABLE  0
#define ENABLE   1
#define DE_RE  2
#define USING_SOFTWARESERIAL  4
#define ID  1

int8_t Timer_Flag = 0,  Data_Print = DISABLE;
uint16_t au16data[2];
int16_t Object, Ambient;

SoftwareSerial mySerial(3, 5);                      // RX 3, TX 5
Modbus master(0,USING_SOFTWARESERIAL,DE_RE);          // Modbus Master, 4:Using SoftwareSerial , DE/RE 2
modbus_t telegram;                                   // Master query structure

void setup() {
  Serial.begin(9600);                                // for Serial Monitor (Ctrl + Shift + M )
  
  master.begin(&mySerial, 19200);                     // begin the ModBus object.
  master.setTimeOut( 25 );                            // Modbus timeout : 25 ms
  
  Serial.println("Waiting for sensor initialization time");
  delay(1000);                                        // Wait fot sensor initialization time
  
  MsTimer2::set(500, timerISR);                     // Timer interval : 500ms.
  MsTimer2::start();                                // Timer Start
}

void loop() {
  if(Timer_Flag) {                        // Check timer interrupt
    Timer_Flag = 0;
    Transfer_Data(ID);                    // Request data transmission : Timer cycle 
  }
  if(master.getState() == COM_WAITING) {  // Get modbus master state : wating for answer
    master.poll();                                    
    Data_Print = ENABLE;                              
  }
  if ((master.getState() == COM_IDLE) && (Data_Print == ENABLE)) {       // Get modbus master state : idle
    if(master.getLastError() == 0 ){                 // Get the last error in the protocol processor (0: No error)
      Object = au16data[0];                          
      Ambient = au16data[1];                       
      Serial.print("Object Temp: ");
      Serial.print(float(Object)/10, 1);            // celsius   
      Serial.print("   Ambient Temp: ");
      Serial.println(float(Ambient)/10, 1);  
     
    }
    else if(master.getLastError() == NO_REPLY) {          // Time-out
      Serial.println("No reply.");
    }
    Data_Print = DISABLE;                              
  }
}

void timerISR() {                                    // Timer Interrupt Service Routine
  Timer_Flag = 1;
}

void Transfer_Data(uint8_t uid) {
  if( (uid == 0) &&(uid>200) ) {                    // ID : 1~200 
    uid = 1;                                        
  }
  telegram.u8id = uid;                             // slave ID 
  telegram.u8fct = 3;                              // function code = 03 (Read Holding Registers) 
  telegram.u16RegAdd = 40002;                      // start address in slave 
  telegram.u16CoilsNo = 2;                         // number of elements to read
  telegram.au16reg = au16data;                     // pointer to a memory array in the Arduino 
  master.query( telegram );                        // send query
}


